/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */
#include "percentagelabel.h"

#include <QFont>
#include <QDBusReply>
#include <QtDBus>

#define ORG_UKUI_STYLE "org.ukui.style"
#define STYLE_NAME "styleName"
#define SYSTEM_FONT_SIZE "systemFontSize"

PercentageLabel::PercentageLabel()
{
    setFontsize();
    watchGsetting();
}

void PercentageLabel::setFontsize()
{
    qDebug() << "m_sysFontSize:" << m_sysFontSize;
    ft.setPointSize(m_sysFontSize * 2);
    this->setFont(ft);
}

void PercentageLabel::watchGsetting()
{
    const QByteArray styleId(ORG_UKUI_STYLE);
    if (QGSettings::isSchemaInstalled(styleId)) {
        QGSettings *styleSettings = new QGSettings(styleId);
        connect(styleSettings, &QGSettings::changed, this, [=](const QString &key) {
            if (key == SYSTEM_FONT_SIZE) {
                m_sysFontSize = styleSettings->get(SYSTEM_FONT_SIZE).toInt();
                setFontsize();
            }
        });
        m_sysFontSize = styleSettings->get(SYSTEM_FONT_SIZE).toInt();
    }
}
