/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */
#include "compatibleinterface.h"
#include <QtDebug>

#define BRIGHTNESS           "brightnessAc"
#define GET_BRIGHTNESS       "GetBrightness"
#define BRIGHTNESS_VALUE     "brightness"
CompatibleInterface::CompatibleInterface(QObject *parent) : QObject(parent)
{
    const QByteArray brightSettings(BRIGHTNESS_SCHEMA);
    brightnesssettings = new QGSettings(brightSettings, QByteArray(), this);

    interface = new QDBusInterface("org.ukui.powermanagement",
                         "/",
                         "org.ukui.powermanagement.interface",
                         QDBusConnection::systemBus());

    connect(brightnesssettings, &QGSettings::changed, [=](QString key) {
        if (BRIGHTNESS == key) {
            brightnes=brightnesssettings->get(BRIGHTNESS).toInt();
            interface->call("RegulateBrightness",brightnes);
            qDebug()<<"brightness"<<brightnes;
        }
    });
}

CompatibleInterface::~CompatibleInterface()
{
    delete iface;
    delete interface;
}
